@@ -2,9 +2,9 @@
  * This is the qmlscene plugin that collects all the Python plugins it can
  * find.
  *
- * Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+ * Copyright (c) 2024 Riverbank Computing Limited <info@riverbankcomputing.com>
  * 
- * This file is part of PyQt5.
+ * This file is part of PyQt6.
  * 
  * This file may be used under the terms of the GNU General Public License
  * version 3.0 as published by the Free Software Foundation and appearing in
@@ -27,6 +27,7 @@
 
 #include <QCoreApplication>
 #include <QDir>
+#include <QLibrary>
 #include <QLibraryInfo>
 #include <QVector>
 #include <QQmlEngine>
@@ -35,27 +36,58 @@
 
 
 // Construct the C++ plugin.
-PyQt5QmlPlugin::PyQt5QmlPlugin(QObject *parent) : QQmlExtensionPlugin(parent),
+PyQt6QmlPlugin::PyQt6QmlPlugin(QObject *parent) : QQmlExtensionPlugin(parent),
         py_plugin_obj(0), sip(0)
 {
     // Make sure the interpreter is initialised.
     if (!Py_IsInitialized())
     {
-        Py_Initialize();
+        QLibrary library(PYTHON_LIB);
 
-        getSipAPI();
+        library.setLoadHints(QLibrary::ExportExternalSymbolsHint);
+
+        if (library.load())
+        {
+            // If we seem to be running in a venv the set the program name
+            // explicitly so that Python finds the right site-packages.
+            QString venv = QString::fromLocal8Bit(qgetenv("VIRTUAL_ENV"));
+
+            if (!venv.isEmpty())
+            {
+                venv.append(QDir::separator());
+#if defined(Q_OS_WIN)
+                venv.append(QLatin1String("Scripts"));
+#else
+                venv.append(QLatin1String("bin"));
+#endif
+                venv.append(QDir::separator()).append(QLatin1String("python"));
+
+                wchar_t *venv_wc = new wchar_t[venv.length() + 1];
+                venv_wc[venv.toWCharArray(venv_wc)] = L'\0';
+
+                Py_SetProgramName(venv_wc);
+
+                delete[] venv_wc;
+            }
+
+            Py_Initialize();
+
+            getSipAPI();
 
 #ifdef WITH_THREAD
-        // Make sure we don't have the GIL.
-        PyEval_InitThreads();
-        PyEval_SaveThread();
+            // Make sure we don't have the GIL.
+#if PY_VERSION_HEX < 0x03070000
+            PyEval_InitThreads();
 #endif
+            PyEval_SaveThread();
+#endif
+        }
     }
 }
 
 
 // Destroy the C++ plugin.
-PyQt5QmlPlugin::~PyQt5QmlPlugin()
+PyQt6QmlPlugin::~PyQt6QmlPlugin()
 {
     if (Py_IsInitialized())
     {
@@ -73,7 +105,7 @@ PyQt5QmlPlugin::~PyQt5QmlPlugin()
 
 
 // Initialize the QML engine.
-void PyQt5QmlPlugin::initializeEngine(QQmlEngine *engine, const char *uri)
+void PyQt6QmlPlugin::initializeEngine(QQmlEngine *engine, const char *uri)
 {
     if (!Py_IsInitialized() || !py_plugin_obj || !sip)
         return;
@@ -108,24 +140,9 @@ void PyQt5QmlPlugin::initializeEngine(QQmlEngine *engine, const char *uri)
             if (res_obj != Py_None)
             {
                 if (res_obj)
-#if PY_MAJOR_VERSION >= 3
                     PyErr_Format(PyExc_TypeError,
                             "unexpected result from initializeEngine(): %S",
                             res_obj);
-#else
-                {
-                    PyObject *res_s = PyObject_Str(res_obj);
-
-                    if (res_s != NULL)
-                    {
-                        PyErr_Format(PyExc_TypeError,
-                                "unexpected result from initializeEngine(): %s",
-                                PyString_AsString(res_s));
-
-                        Py_DECREF(res_s);
-                    }
-                }
-#endif
 
                 td = 0;
             }
@@ -144,7 +161,7 @@ void PyQt5QmlPlugin::initializeEngine(QQmlEngine *engine, const char *uri)
 
 
 // Register the plugin's types.
-void PyQt5QmlPlugin::registerTypes(const char *uri)
+void PyQt6QmlPlugin::registerTypes(const char *uri)
 {
     // Construct the import path that the engine will have used by default.
     // Unfortunately we won't know about any directories added by qmlscene's -I
@@ -163,13 +180,13 @@ void PyQt5QmlPlugin::registerTypes(const char *uri)
         QLatin1Char sep(':');
 #endif
 
-        QStringList env_dirs = QString::fromLatin1(env_path).split(sep, QString::SkipEmptyParts);
+        QStringList env_dirs = QString::fromLatin1(env_path).split(sep, Qt::SkipEmptyParts);
 
         foreach (QString dir_name, env_dirs)
             import_path << QDir(dir_name).canonicalPath();
     }
 
-    import_path << QLibraryInfo::location(QLibraryInfo::Qml2ImportsPath);
+    import_path << QLibraryInfo::path(QLibraryInfo::Qml2ImportsPath);
 
     // Try and find the URI directory, check that it has a qmldir file and that
     // it has a file called *plugin.py.
@@ -223,7 +240,7 @@ void PyQt5QmlPlugin::registerTypes(const char *uri)
 
 
 // Call the registerTypes() function of a plugin.
-bool PyQt5QmlPlugin::callRegisterTypes(const QString &py_plugin,
+bool PyQt6QmlPlugin::callRegisterTypes(const QString &py_plugin,
         const char *uri)
 {
     // Import the plugin.
@@ -234,7 +251,7 @@ bool PyQt5QmlPlugin::callRegisterTypes(const QString &py_plugin,
 
     // Get QQmlExtensionPlugin.  We do this after the import of the plugin to
     // allow it to fiddle with the context.
-    PyObject *extension_plugin = getModuleAttr("PyQt5.QtQml",
+    PyObject *extension_plugin = getModuleAttr("PyQt6.QtQml",
             "QQmlExtensionPlugin");
 
     if (!extension_plugin)
@@ -304,14 +321,13 @@ bool PyQt5QmlPlugin::callRegisterTypes(const QString &py_plugin,
 
 
 // Add a directory to sys.path.
-bool PyQt5QmlPlugin::addToSysPath(const QString &py_plugin_dir)
+bool PyQt6QmlPlugin::addToSysPath(const QString &py_plugin_dir)
 {
     PyObject *sys_path = getModuleAttr("sys", "path");
 
     if (!sys_path)
         return false;
 
-#if PY_VERSION_HEX >= 0x03030000
     PyObject *plugin_dir_obj = PyUnicode_FromKindAndData(PyUnicode_2BYTE_KIND,
             py_plugin_dir.constData(), py_plugin_dir.length());
 
@@ -320,43 +336,6 @@ bool PyQt5QmlPlugin::addToSysPath(const QString &py_plugin_dir)
         Py_DECREF(sys_path);
         return false;
     }
-#elif PY_VERSION_HEX >= 0x03000000
-#if defined(Py_UNICODE_WIDE)
-    QVector<uint> ucs4 = py_plugin_dir.toUcs4();
-
-    PyObject *plugin_dir_obj = PyUnicode_FromUnicode(0, ucs4.size());
-
-    if (!plugin_dir_obj)
-    {
-        Py_DECREF(sys_path);
-        return false;
-    }
-
-    memcpy(PyUnicode_AS_UNICODE(plugin_dir_obj), ucs4.constData(),
-            ucs4.size() * sizeof (Py_UNICODE));
-#else
-    PyObject *plugin_dir_obj = PyUnicode_FromUnicode(0,
-            py_plugin_dir.length());
-
-    if (!plugin_dir_obj)
-    {
-        Py_DECREF(sys_path);
-        return false;
-    }
-
-    memcpy(PyUnicode_AS_UNICODE(plugin_dir_obj), py_plugin_dir.utf16(),
-            py_plugin_dir.length() * sizeof (Py_UNICODE));
-#endif
-#else
-    PyObject *plugin_dir_obj = PyString_FromString(
-            py_plugin_dir.toLatin1().constData());
-
-    if (!plugin_dir_obj)
-    {
-        Py_DECREF(sys_path);
-        return false;
-    }
-#endif
 
     int rc = PyList_Append(sys_path, plugin_dir_obj);
     Py_DECREF(plugin_dir_obj);
@@ -372,7 +351,7 @@ bool PyQt5QmlPlugin::addToSysPath(const QString &py_plugin_dir)
 
 
 // Return the named attribute object from the named module.
-PyObject *PyQt5QmlPlugin::getModuleAttr(const char *module, const char *attr)
+PyObject *PyQt6QmlPlugin::getModuleAttr(const char *module, const char *attr)
 {
     PyObject *mod = PyImport_ImportModule(module);
 
@@ -388,19 +367,9 @@ PyObject *PyQt5QmlPlugin::getModuleAttr(const char *module, const char *attr)
 
 
 // Get the SIP API.
-void PyQt5QmlPlugin::getSipAPI()
+void PyQt6QmlPlugin::getSipAPI()
 {
-#if defined(SIP_USE_PYCAPSULE)
-    sip = (const sipAPIDef *)PyCapsule_Import("sip._C_API", 0);
-#else
-    PyObject *c_api = getModuleAttr("sip", "_C_API");
-
-    if (c_api)
-    {
-        sip = (const sipAPIDef *)PyCObject_AsVoidPtr(c_api);
-        Py_DECREF(c_api);
-    }
-#endif
+    sip = (const sipAPIDef *)PyCapsule_Import("PyQt6.sip._C_API", 0);
 
     if (!sip)
         PyErr_Print();
