@@ -1,8 +1,8 @@
-// This defines the interfaces to the Q_ENUMS and Q_FLAGS support.
+// This defines the interfaces to the pyqtEnum support.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2024 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -23,23 +23,20 @@
 
 
 #include <QByteArray>
-#include <QHash>
 #include <QList>
+#include <QPair>
 
 
-struct EnumsFlags
+// Encapsulate the definition of a Python enum as required by QMetaEnum.
+struct EnumFlag
 {
-    EnumsFlags(const char *name_, bool isFlag_) : name(name_), isFlag(isFlag_)
-    {
-    }
-
     QByteArray name;
     bool isFlag;
-    QHash<QByteArray, int> keys;
+    QList<QPair<QByteArray, int> > keys;
 };
 
 
-QList<EnumsFlags> qpycore_get_enums_flags_list();
+EnumFlag qpycore_pop_enum_flag(PyObject *enum_cls);
 
 
 #endif
