@@ -1,8 +1,8 @@
 // This contains the implementation of various odds and ends.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2024 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -20,24 +20,40 @@
 
 #include <Python.h>
 
+#include <QByteArray>
+
 #include "qpycore_api.h"
 #include "qpycore_misc.h"
-#include "qpycore_types.h"
 
 #include "sipAPIQtCore.h"
 
 
-// Return true if the given type (which must be a class) was wrapped for PyQt5.
-bool qpycore_is_pyqt_class(const sipTypeDef *td)
+// Convert a borrowed ASCII string object to a QByteArray.
+QByteArray qpycore_convert_ASCII(PyObject *str_obj)
 {
-    return PyType_IsSubtype(Py_TYPE(sipTypeAsPyTypeObject(td)),
-            &qpycore_pyqtWrapperType_Type);
+    const char *str = sipString_AsASCIIString(&str_obj);
+
+    if (!str)
+    {
+        Py_DECREF(str_obj);
+        return QByteArray();
+    }
+
+    QByteArray ba(str);
+
+    Py_DECREF(str_obj);
+
+    return ba;
+}
+
+
+// Return true if the given type was wrapped for PyQt.
+bool qpycore_is_pyqt_type(const sipTypeDef *td)
+{
+    return sipCheckPluginForType(td, "PyQt6.QtCore");
 }
 
 
-#if PY_MAJOR_VERSION >= 3
-// Python v3 doesn't have the Unicode equivalent of Python v2's
-// PyString_ConcatAndDel().
 void qpycore_Unicode_ConcatAndDel(PyObject **string, PyObject *newpart)
 {
     PyObject *old = *string;
@@ -54,4 +70,3 @@ void qpycore_Unicode_ConcatAndDel(PyObject **string, PyObject *newpart)
 
     Py_XDECREF(newpart);
 }
-#endif
