@@ -1,8 +1,8 @@
 // This contains the support for Python objects and Qt's metatype system.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2024 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -26,9 +26,6 @@
 #include "sipAPIQtCore.h"
 
 
-// The Qt metatype id.
-int PyQt_PyObject::metatype;
-
 // The current pickle protocol.
 PyObject *qpycore_pickle_protocol;
 
@@ -93,6 +90,14 @@ PyQt_PyObject &PyQt_PyObject::operator=(const PyQt_PyObject &other)
 }
 
 
+// Comparison operator.  It must be global to be available to QMetaType.
+bool operator==(const PyQt_PyObject &self, const PyQt_PyObject &other)
+{
+    // Note that we implement 'self is other' rather than 'self == other'.
+    return self.pyobject == other.pyobject;
+}
+
+
 // Serialise operator.
 QDataStream &operator<<(QDataStream &out, const PyQt_PyObject &obj)
 {
@@ -130,10 +135,10 @@ QDataStream &operator<<(QDataStream &out, const PyQt_PyObject &obj)
 
             if (ser_obj)
             {
-                if (SIPBytes_Check(ser_obj))
+                if (PyBytes_Check(ser_obj))
                 {
-                    ser = SIPBytes_AS_STRING(ser_obj);
-                    len = SIPBytes_GET_SIZE(ser_obj);
+                    ser = PyBytes_AsString(ser_obj);
+                    len = PyBytes_Size(ser_obj);
                 }
                 else
                 {
@@ -143,7 +148,7 @@ QDataStream &operator<<(QDataStream &out, const PyQt_PyObject &obj)
             }
             else
             {
-                PyErr_Print();
+                pyqt6_err_print();
             }
         }
 
@@ -190,7 +195,7 @@ QDataStream &operator>>(QDataStream &in, PyQt_PyObject &obj)
 
         if (loads)
         {
-            PyObject *ser_obj = SIPBytes_FromStringAndSize(ser, len);
+            PyObject *ser_obj = PyBytes_FromStringAndSize(ser, len);
 
             if (ser_obj)
             {
