@@ -1,8 +1,8 @@
 // This contains the definition of the PyQtSlotProxy class.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2024 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -34,7 +34,7 @@
 
 
 QT_BEGIN_NAMESPACE
-class QMutex;
+class QRecursiveMutex;
 QT_END_NAMESPACE
 
 class PyQtSlot;
@@ -60,6 +60,7 @@ public:
     void unislot(void **qargs);
     void disable();
 
+    static void deleteSlotProxy(const QMetaObject::Connection *connection);
     static void deleteSlotProxies(const QObject *transmitter,
             const QByteArray &signal_signature);
     static PyQtSlotProxy *findSlotProxy(const QObject *transmitter,
@@ -77,7 +78,10 @@ public:
     static const QByteArray proxy_slot_signature;
 
     // The mutex around the proxies hash.
-    static QMutex *mutex;
+    static QRecursiveMutex *mutex;
+
+    // The connection to the proxy.
+    QMetaObject::Connection connection;
 
 private:
     // The last QObject sender.
