@@ -1,8 +1,8 @@
 // This is the implementation of the QQmlListProperty class.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2024 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -24,11 +24,16 @@
 #include <QQmlListProperty>
 
 #include "qpyqmllistproperty.h"
+#include "qpyqmllistpropertywrapper.h"
 #include "qpyqml_listdata.h"
 
 #include "sipAPIQtQml.h"
 
 
+// The type object.
+PyTypeObject *qpyqml_QQmlListProperty_TypeObject;
+
+
 // Forward declarations.
 extern "C" {
 static PyObject *QQmlListProperty_call(PyObject *, PyObject *args,
@@ -36,9 +41,9 @@ static PyObject *QQmlListProperty_call(PyObject *, PyObject *args,
 }
 
 static void list_append(QQmlListProperty<QObject> *p, QObject *el);
-static QObject *list_at(QQmlListProperty<QObject> *p, int idx);
+static QObject *list_at(QQmlListProperty<QObject> *p, qsizetype idx);
 static void list_clear(QQmlListProperty<QObject> *p);
-static int list_count(QQmlListProperty<QObject> *p);
+static qsizetype list_count(QQmlListProperty<QObject> *p);
 static void bad_result(PyObject *py_res, const char *context);
 
 
@@ -50,65 +55,36 @@ PyDoc_STRVAR(QQmlListProperty_doc,
 
 // This implements the QQmlListProperty Python type.  It is a sub-type of the
 // standard string type that is callable.
-PyTypeObject qpyqml_QQmlListProperty_Type = {
-    PyVarObject_HEAD_INIT(NULL, 0)
-    SIP_TPNAME_CAST("PyQt5.QtQml.QQmlListProperty"),
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    QQmlListProperty_call,
-    0,
-    0,
-    0,
-    0,
-    Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE|
-#if PY_MAJOR_VERSION >= 3
-    Py_TPFLAGS_UNICODE_SUBCLASS,
-#else
-    Py_TPFLAGS_STRING_SUBCLASS,
-#endif
-    QQmlListProperty_doc,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-    0,
-#if PY_VERSION_HEX >= 0x03040000
-    0,
-#endif
+
+// Define the slots.
+static PyType_Slot qpyqml_QQmlListProperty_Slots[] = {
+    {Py_tp_base,        (void *)&PyUnicode_Type},
+    {Py_tp_call,        (void *)QQmlListProperty_call},
+    {Py_tp_doc,         (void *)QQmlListProperty_doc},
+    {0,                 0}
 };
 
 
+// Define the type.
+static PyType_Spec qpyqml_QQmlListProperty_Spec = {
+    "PyQt6.QtQml.QQmlListProperty",
+    0,
+    0,
+    Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_UNICODE_SUBCLASS,
+    qpyqml_QQmlListProperty_Slots
+};
+
+
+// Initialise the type and return true if there was no error.
+bool qpyqml_QQmlListProperty_init_type()
+{
+    qpyqml_QQmlListProperty_TypeObject = (PyTypeObject *)PyType_FromSpec(
+            &qpyqml_QQmlListProperty_Spec);
+
+    return qpyqml_QQmlListProperty_TypeObject;
+}
+
+
 // The QQmlListProperty init function.
 static PyObject *QQmlListProperty_call(PyObject *, PyObject *args,
         PyObject *kwds)
@@ -124,12 +100,12 @@ static PyObject *QQmlListProperty_call(PyObject *, PyObject *args,
             &py_list, &py_append, &py_count, &py_at, &py_clear))
         return 0;
 
-    // Check the type object.
-    if (!PyObject_TypeCheck(py_type, &PyType_Type))
+    // Check the type is derived from QObject.
+    if (!PyObject_TypeCheck(py_type, &PyType_Type) ||
+        !PyType_IsSubtype((PyTypeObject *)py_type, sipTypeAsPyTypeObject(sipType_QObject)))
     {
         PyErr_Format(PyExc_TypeError,
-                "type argument must be of type 'type', not '%s'",
-                Py_TYPE(py_type)->tp_name);
+                "type argument must be a sub-type of QObject");
         return 0;
     }
 
@@ -142,7 +118,7 @@ static PyObject *QQmlListProperty_call(PyObject *, PyObject *args,
     {
         PyErr_Format(PyExc_TypeError,
                 "object argument must be of type 'QObject', not '%s'",
-                Py_TYPE(py_obj)->tp_name);
+                sipPyTypeName(Py_TYPE(py_obj)));
         return 0;
     }
 
@@ -158,10 +134,9 @@ static PyObject *QQmlListProperty_call(PyObject *, PyObject *args,
     ListData *list_data = new ListData(py_type, py_obj, py_list, py_append,
             py_count, py_at, py_clear, obj);
 
-    // Create the C++ QDeclarativeListProperty<QObject> with the list data as
-    // the data.  Note that we will create a new one each time the property
-    // getter is called - however, under normal usage, this should only be
-    // once.  Also note that the callables will not be reached by the garbage
+    // Create the C++ QQmlListProperty<QObject> with the list data as the data.
+    // Note that we will create a new one each time the property getter is
+    // called.  Also note that the callables will not be reached by the garbage
     // collector.
     QQmlListProperty<QObject> *prop = new QQmlListProperty<QObject>(obj,
             list_data, ((py_list || py_append) ? list_append : 0),
@@ -170,15 +145,7 @@ static PyObject *QQmlListProperty_call(PyObject *, PyObject *args,
             ((py_list || py_clear) ? list_clear : 0));
 
     // Convert it to a Python object.
-    static const sipTypeDef *mapped_type = 0;
-
-    if (!mapped_type)
-        mapped_type = sipFindType("QQmlListProperty<QObject>");
-
-    Q_ASSERT(mapped_type);
-
-    // Make sure ownership is with Python.
-    PyObject *prop_obj = sipConvertFromNewType(prop, mapped_type, py_obj);
+    PyObject *prop_obj = qpyqml_QQmlListPropertyWrapper_New(prop, py_list);
 
     if (!prop_obj)
     {
@@ -207,8 +174,8 @@ static void list_append(QQmlListProperty<QObject> *p, QObject *el)
         {
             PyErr_Format(PyExc_TypeError,
                 "list element must be of type '%s', not '%s'",
-                    ((PyTypeObject *)ldata->py_type)->tp_name,
-                    Py_TYPE(py_el)->tp_name);
+                    sipPyTypeName(((PyTypeObject *)ldata->py_type)),
+                    sipPyTypeName(Py_TYPE(py_el)));
         }
         else if (ldata->py_list)
         {
@@ -236,16 +203,16 @@ static void list_append(QQmlListProperty<QObject> *p, QObject *el)
     }
 
     if (!ok)
-        PyErr_Print();
+        pyqt6_qtqml_err_print();
 
     SIP_UNBLOCK_THREADS
 }
 
 
 // Get the length of the list.
-static int list_count(QQmlListProperty<QObject> *p)
+static qsizetype list_count(QQmlListProperty<QObject> *p)
 {
-    int res = -1;
+    qsizetype res = -1;
 
     SIP_BLOCK_THREADS
 
@@ -263,9 +230,7 @@ static int list_count(QQmlListProperty<QObject> *p)
 
         if (py_res)
         {
-            PyErr_Clear();
-
-            res = SIPLong_AsLong(py_res);
+            res = sipLong_AsInt(py_res);
 
             if (PyErr_Occurred())
             {
@@ -280,7 +245,7 @@ static int list_count(QQmlListProperty<QObject> *p)
 
     if (res < 0)
     {
-        PyErr_Print();
+        pyqt6_qtqml_err_print();
         res = 0;
     }
 
@@ -291,7 +256,7 @@ static int list_count(QQmlListProperty<QObject> *p)
 
 
 // Get an item from the list.
-static QObject *list_at(QQmlListProperty<QObject> *p, int idx)
+static QObject *list_at(QQmlListProperty<QObject> *p, qsizetype idx)
 {
     QObject *qobj = 0;
 
@@ -330,7 +295,7 @@ static QObject *list_at(QQmlListProperty<QObject> *p, int idx)
     }
 
     if (!qobj)
-        PyErr_Print();
+        pyqt6_qtqml_err_print();
 
     SIP_UNBLOCK_THREADS
 
@@ -369,7 +334,7 @@ static void list_clear(QQmlListProperty<QObject> *p)
     }
 
     if (!ok)
-        PyErr_Print();
+        pyqt6_qtqml_err_print();
 
     SIP_UNBLOCK_THREADS
 }
@@ -378,18 +343,6 @@ static void list_clear(QQmlListProperty<QObject> *p)
 // Raise an exception for an unexpected result.
 static void bad_result(PyObject *py_res, const char *context)
 {
-#if PY_MAJOR_VERSION >= 3
     PyErr_Format(PyExc_TypeError, "unexpected result from %s function: %S",
             context, py_res);
-#else
-    PyObject *py_res_s = PyObject_Str(py_res);
-
-    if (py_res_s != NULL)
-    {
-        PyErr_Format(PyExc_TypeError, "unexpected result from %s function: %s",
-                context, PyString_AsString(py_res_s));
-
-        Py_DECREF(py_res_s);
-    }
-#endif
 }
