@@ -1,8 +1,8 @@
 // This is the definition of the QQmlListProperty class.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2024 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -25,12 +25,11 @@
 #include <Python.h>
 
 
-extern "C" {
+// The type object.
+extern PyTypeObject *qpyqml_QQmlListProperty_TypeObject;
 
-// This implements the QQmlListProperty type.
-extern PyTypeObject qpyqml_QQmlListProperty_Type;
 
-}
+bool qpyqml_QQmlListProperty_init_type();
 
 
 #endif
